#pragma rtGlobals=3		// Use modern global access method.

//Functions listed here are Explicitly for version 2.1 of Ellis.  Each function has a setup function and an executable fit function.
Menu "Ellis2" 
	Submenu "1. Run your setup function"
		"1.1 Setup Randles", setupZrandles()
		"1.2 Setup QPE", setupZqandles()
		"1.3 Setup QPE + Warburg", setupZqandlesW()
		"1.4 Setup Randles + Warburg", setupZrandlesW()
		"1.5 Setup Randles2", setupZrandles2()
		"1.6 Setup Qandles2", setupZqandles2()
		"1.7 Setup Qandles + Porous Bounded Warburg", setupZqandlesB()
		"1.8 Setup Qandles + Bounded Warburg", setupZqandlesT()
		"1.9 Setup Qandles2 parallel", setupZqandles2P()
		"1.10 Setup Reduced transmission line", setupZunifiedtline0()
		"1.11 Setup balanced transmission line", setupZunifiedtlineC()
//		"1.12 Jean",setupZ
		"1.X Setup user function", Execute/q "setup"+testsetupvals[0]+"()"
	End
End

Function setupZrandles()

	Make/O/D limits ={{0,0,0},{1e3,0.1,1e8}}		// make the lower and upper limits
	Make/O/D coefs = {100,0.000001,3e3}			// make the initial coefficients
	make/T/O coeffnames={"RS","Cdl","RP"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zrandles"
	wave/t testsetupvals
	testsetupvals[0]="Zrandles"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zrandles(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw//declare wave names.  Don't edit this
	wave/c complexer // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Cdl //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Cdl=w[1]
	Rp=w[2]

	// Calculate your function here.  Make sure to keep track of temporary waves and variables.

	complexer[0,midpointt-1] = Rs+1/(1/Rp+1/(ki/-(Cdl*freqw)))  //Compute the final value of the complex function here.
	complexcat[0,midpointt-1]=real(complexer[p]) // don't edit this line.  Writes real part to complexcat first half
	complexcat[midpointt, ]=imag(complexer[p-midpointt]) //don't edit this line.  Writes imaginary part to complexcat second part
End

Function setupZrandlesW()
	Make/O/D limits ={{0,0,0,0},{1e5,0.1,1e6,1000}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,5e4}			// make the initial coefficients
	make/T/O coeffnames={"RS","Cdl","RP","sigmaW"}	// make a list of coefficient names
	string/g currentmodel="ZrandlesW"
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	wave/t testsetupvals
	testsetupvals[0]="ZrandlesW"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZrandlesW(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki,kii //pull in complex constants.  Note that ki = i and kii = 1-i
	variable Rs, Rp, Cdl,sigmaW //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Cdl=w[1]
	Rp=w[2]
	sigmaW=w[3]

	complexer[0,midpointt-1] = Rs+1/(1/(Rp+sigmaW*kii/sqrt(freqw))+1/(ki/-(Cdl*freqw)))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandles()
	Make/O/D limits ={{0,0,0,0},{100,0.1,1e6,1}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,5e4,0.8}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zqandles"
	wave/t testsetupvals
	testsetupvals[0]="Zqandles"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End


Threadsafe Function Zqandles(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw 
	wave/c complexer
	variable/g  midpointt
	variable/g/c ki
	variable Rs, Rp, Qdl,n 
	
	Rs=w[0]
	Qdl=w[1]
	Rp=w[2]
	n=w[3]

	complexer[0,midpointt-1]=Rs+1/(1/Rp+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandlesW()
	Make/O/D limits ={{0,0,0,0,0},{100,0.1,1e6,1,1000}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.0001,5e5,0.8,100}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","sigmaW"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesW"
	wave/t testsetupvals
	testsetupvals[0]="ZqandlesW"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesW(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki, kii //pull in complex constant
	variable Rs, Rp, Qdl,n,sigmaW //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	sigmaW=w[4]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+sigmaW*kii/sqrt(freqw))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZrandles2()
	Make/O/D limits ={{0,0,0,0,0},{1e5,0.1,1e6,0.1,1e6}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,0.00001,5e5}			// make the initial coefficients
	make/T/O coeffnames={"RS","C1","R1","C2","R2"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zrandles2"
	wave/t testsetupvals
	testsetupvals[0]="Zrandles2"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zrandles2(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs, R1, C1, R2, C2 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	C1=w[1]
	R1=w[2]
	C2=w[3]
	R2=w[4]

	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(-ki/(C2*freqw))))+1/(ki/-(C1*freqw)))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandles2()
	Make/O/D limits ={{0,0,0,0,0,0,0},{1e5,0.1,1e6,0.1,1e6,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,0.00001,5e5,0.8,0.8}			// make the initial coefficients
	make/T/O coeffnames={"RS","Q1","R1","Q2","R2","n1","n2"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zqandles2"
	wave/t testsetupvals
	testsetupvals[0]="Zqandles2"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zqandles2(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs, R1, Q1, R2, Q2, n1, n2 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Q1=w[1]
	R1=w[2]
	Q2=w[3]
	R2=w[4]
	n1=w[5]
	n2=w[6]

	complexer[0,midpointt-1]=Rs+1/(1/R1+Q1*(ki*freqw)^n1)+1/(1/R2+Q2*(Ki*freqw)^n2) //Rs-R1||Q1-R2||Q2
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(1/(Q2*(ki*freqw)^n2))))+1/(1/(Q1*(ki*freqw)^n2)))
//	complexer[0,midpointt-1]=Rs+1/(1/(Q1*(ki*freqw)^n1)+1/(R1+1/(1/R2+1/(Q2*(ki*freqw)^n2))))
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(ki/-(Q1*freqw)^n1)))+1/(ki/-(Q2*freqw)^n2))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandles2P()
	Make/O/D limits ={{0,0,0,0,0,0,0},{1e5,0.1,1e6,0.1,1e6,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {5e4,0.0001,5e5,0.00001,5e5,0.8,0.8}			// make the initial coefficients
	make/T/O coeffnames={"RS","Q1","R1","Q2","R2","n1","n2"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="Zqandles2P"
	wave/t testsetupvals
	testsetupvals[0]="Zqandles2P"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function Zqandles2P(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs, R1, Q1, R2, Q2, n1, n2 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Q1=w[1]
	R1=w[2]
	Q2=w[3]
	R2=w[4]
	n1=w[5]
	n2=w[6]
	complexer[0,midpointt-1]=Rs+1/(1/(Q1*(ki*freqw)^n1) + 1/(R1 + 1/(1/R2 + 1/(Q2*(ki*freqw)^n2))))
//	complexer[0,midpointt-1]=Rs+1/(1/R1+Q1*(ki*freqw)^n1)+1/(1/R2+Q2*(Ki*freqw)^n2) //Rs-R1||Q1-R2||Q2
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(1/(Q2*(ki*freqw)^n2))))+1/(1/(Q1*(ki*freqw)^n2)))
//	complexer[0,midpointt-1]=Rs+1/(1/(Q1*(ki*freqw)^n1)+1/(R1+1/(1/R2+1/(Q2*(ki*freqw)^n2))))
//	complexer[0,midpointt-1]=Rs+1/(1/(R1+1/(1/R2+1/(ki/-(Q1*freqw)^n1)))+1/(ki/-(Q2*freqw)^n2))
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZunifiedtline()
	wave freqdata
	Make/O/D limits ={{0,0,0,0,0,0,0,0,0,0},{1,50000,.01,.01,1,1,1,1,50000,50}}		// make the lower and upper limits
	Make/O/D coefs = {.05,5000,.00001,.00000001,.9,.9,.000001,.05,300,100}			// make the initial coefficients
	make/T/O coeffnames={"r1","r2","qa1","qa2","na1","na2","L","ra","rb","rs"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	make/o/c/n=(numpnts(freqdata)) complexer
	string/g currentmodel="Zunifiedtline"
	wave/t testsetupvals
//	wave/c ksi,za,lam,slam,clam
	testsetupvals[0]="Zunifiedtline"
	make/o/c/d/n=(numpnts(freqdata)) ksi
	make/o/c/d/n=(numpnts(freqdata)) za
	make/o/c/d/n=(numpnts(freqdata)) lam
	make/o/c/d/n=(numpnts(freqdata)) slam
	make/o/c/d/n=(numpnts(freqdata)) clam
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Function setupZunifiedtline0()
	wave freqdata
	Make/O/D limits ={{0,0,0,0,0,0,0,0,0,0,0,0,0},{100,1000,1,1e6,.01,1,1e6,.01,1,10000,.01,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {50,500,.9,1e4,.00001,.9,1e4,.00001,100,.9,.00001,.9,.001}			// make the initial coefficients
	make/T/O coeffnames={"Rs","r1","r2","ra","qa","na","rb","qb","nb","rk","qk","nk","L"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	make/o/c/n=(numpnts(freqdata)) cx
	string/g currentmodel="Zunifiedtline0"
	wave/t testsetupvals
//	wave/c ksi,za,lam,slam,clam
	testsetupvals[0]="Zunifiedtline0"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Function Zunifiedtline0(w,complexcat,u): fitfunc
	wave complexcat, w, u//declare wave names.  Don't edit this
	wave/c cx // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
	variable o = midpointt
	variable/g/c ki//pull in complex constant
	variable/c j=ki
	variable a,b,k,c,d,g,h,i,x,y,s,m,L
	
	// assumes that x2<<x1 | x2=0

	s=w[0]//	Rs
	x=w[1]//	r1
	y=w[2]//	r2
	a=w[3]//	ra
	g=w[4]//	qa
	c=w[5]//	na
	b=w[6]//	rb
	h=w[7]//	qb
	d=w[8]//	nb
	k=w[9]//	rk
	i=w[10]//	qk
	m=w[11]//	nk
	L=w[12]//	L


	cx[0,o-1]=s+(a*(b*cosh((L*sqrt(x+i*k*(j*u)^m*x))/sqrt(k))+(1+b*h*(j*u)^d)*x*sqrt(k/(x+i*k*(j*u)^m*x))*sinh(L/sqrt(k/(x+i*k*(j*u)^m*x)))))/((1+a*g*(j*u)^c)*(1+b*h*(j*u)^d)*((a/(1+a*g*(j*u)^c)+b/(1+b*h*(j*u)^d))*cosh((L*sqrt(x+i*k*(j*u)^m*x))/sqrt(k))+((k/(1+i*k*(j*u)^m)+(a*b)/((1+a*g*(j*u)^c)*(1+b*h*(j*u)^d)*x))*sqrt(x+i*k*(j*u)^m*x)*sinh((L*sqrt(x+i*k*(j*u)^m*x))/sqrt(k)))/sqrt(k)))
	complexcat[0,o-1]=real(cx[p]) // don't edit this line.  Writes real part to complexcat first half
	complexcat[o, ]=imag(cx[p-o]) //don't edit this line.  Writes imaginary part to complexcat second part
End

//Function Zunifiedtline(w,complexcat,freqw): fitfunc
//	wave complexcat, w, freqw//declare wave names.  Don't edit this
//	wave/c complexer // don't edit this.  Note that you must have a properly scaled complexer wave in your data directory.
//	variable/g midpointt //pull in midpoint value.  This is what divides the real and imaginary parts.
//	variable/g/c ki//pull in complex constant
//	variable r1,rs,qa1,qa2,na1,na2,L,r2,ra,rb//declare variable names; you can edit this
//	variable x1,x2,zb
//	wave/c ksi,za,lam
//	variable o = midpointt
//	
//	
//	r1=w[0] //assign your variable names to the solution wave.  Edit this.
//	r2=w[1]
//	qa1=w[2]
//	qa2=w[3]
//	na1=w[4]
//	na2=w[5]
//	L=w[6]
//	r3=w[7]
//	Rs=w[8]
//	r4=w[9]
//
//	
//	Za = ra/(1+ra*qa*(ki*freqw)^na
//	Zb = rb/(1+rb*qb*(ki*freqw)^nb
//	Zk = rk/(1+rk*qk*(ki*freqw)^nk
////	lam=sqrt(ksi/x1)
////	lam = sqrt(ksi/(x1 + x2))
////	slam=sinh(L/lam)
////	clam=cosh(L/lam)	
////	complexer[0,midpointt-1]=Rs+(lam*slam*x1^2*Za+clam*x1*Za*Zb)/(x1*(lam*slam*x1+(slam*Za*Zb)/(lam*x1)+clam*(Za+Zb))) //such that x2=r2=0
//	complexer[0,midpointt-1]=Rs+(sqrt(ksi/x1)*sinh(L/sqrt(ksi/x1))*x1^2*Za+cosh(L/sqrt(ksi/x1))*x1*Za*Zb)/(x1*(sqrt(ksi/x1)*sinh(L/sqrt(ksi/x1))*x1+(sinh(L/sqrt(ksi/x1))*Za*Zb)/(sqrt(ksi/x1)*x1)+cosh(L/sqrt(ksi/x1))*(Za+Zb)))
////	complexer[0,midpointt-1]=Rs+(lam*sinh(L/lam)*x1^2*Za+cosh(L/lam)*x1*Za*Zb)/(x1*(lam*sinh(L/lam)*x1+(sinh(L/lam)*Za*Zb)/(lam*x1)+cosh(L/lam)*(Za+Zb)))
////	complexer[0,midpointt-1]=Rs+(L*x1*x2*(lam^2*slam*(x1+x2)^2+slam*Za*Zb+clam*lam*(x1+x2)*(Za+Zb))+lam*((2*x1*x2+clam*(x1^2+x2^2))*Za*Zb+lam*slam*(x1+x2)*(x1^2*Za+x2^2*Zb)))/((x1+x2)*(lam^2*Slam*(x1+x2)^2+slam*Za*Zb+clam*lam*(x1+x2)*(Za+Zb)))
////	complexer[0,midpointt-1]=rs+(L*lam*x1*x2*(x1+x2)*sinh(L/lam)+x1*(lam*x1*sinh(L/lam)+L*x2*cosh(L/lam))*za+x2*(lam*x2*sinh(L/lam)+L*x1*cosh(L/lam))*zb+(2*x1*x2+(x1^2+x2^2)*cosh(L/lam)+L*x1*x2*sinh(L/lam)/lam)*za*zb/(x1+x2))/(x1+x2)/(lam*(x1+x2)*sinh(L/lam)+(za+zb)*cosh(L/lam)+Za*Zb*sinh(L/lam)/(lam*(x1+x2)))
////	complexer[0,midpointt-1]=(L*lam*x1*x2*(x1+x2)*slam+x1*(lam*x1*slam+L*x2*clam)*za+x2*(lam*x2*slam+L*x1*clam)*zb+(2*x1*x2+(x1^2+x2^2)*clam+L*x1*x2*slam/lam)*za*zb/(x1+x2))/(x1+x2)/(lam*(x1+x2)*slam+(za+zb)*clam+Za*Zb*slam/(lam*(x1+x2)))
////	complexer[0,midpointt-1] =1/(x1 + x2)/(lam*(x1 + x2)*sinh(L/lam) + (za + zb)*cosh(L/lam) + za*zb*sinh(L/lam)/(sinh(L/lam)*(x1 + x2)))*(L*lam*x1*x2*(x1 + x2)*sinh(L/lam) + x1*(lam*x1*sinh(L/lam) + L*x2*cosh(L/lam))*za + (2*x1*x2 + (x1^2 + x2^2)*cosh(L/lam) + L/lam*x1*x2*sinh(L/lam))*za*zb/(x1 + x2))
//	complexcat[0,midpointt-1]=real(complexer[p]) // don't edit this line.  Writes real part to complexcat first half
//	complexcat[midpointt, ]=imag(complexer[p-midpointt]) //don't edit this line.  Writes imaginary part to complexcat second part
//End

Function setupZqandlesB()
	Make/O/D limits ={{0,0,0,0,0,0},{100,0.1,1e6,1,1,1e7}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,1e3,0.8,1e-4,1e6}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","B","Y0"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesB"
	wave/t testsetupvals
	testsetupvals[0]="ZqandlesB"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesB(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Qdl,n,B,Y0 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	B=w[4]
	Y0=w[5]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+1/Y0/sqrt(ki*freqw)*tanh(B*sqrt(ki*freqw)))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandlesT()
	Make/O/D limits ={{0,0,0,0,0,0},{100,0.1,1e6,1,1,1e7}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,1e3,0.8,1e-4,1e6}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","B","Y0"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesT"
	wave/t testsetupvals
	testsetupvals[0]="ZqandlesT"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesT(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Qdl,n,B,Y0 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	B=w[4]
	Y0=w[5]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+1/Y0/sqrt(ki*freqw)*coth(B*sqrt(ki*freqw)))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZqandlesG()
	Make/O/D limits ={{0,0,0,0,0,0},{100,0.1,1e6,1,1,1e7}}		// make the lower and upper limits
	Make/O/D coefs = {50,0.00001,1e3,0.8,1e-4,1e6}			// make the initial coefficients
	make/T/O coeffnames={"RS","Qdl","RP","n","k","Y0"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZqandlesG"
	wave/t testsetupvals
	testsetupvals[0]="ZrandlesG"
	make/t/o tempwavenames={"complexer"}
	preflight1()
End

Threadsafe Function ZqandlesG(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/d/c complexer //pull in temporary complex wave
	wave/d realout, imagout
	variable/g  midpointt //pull in midpoint
	variable/g/c ki//pull in complex constant
	variable Rs, Rp, Qdl,n,k0,Y0 //declare variable names; you can edit this
	
	Rs=w[0] //assign your variable names to the solution wave.  Edit this.
	Qdl=w[1]
	Rp=w[2]
	n=w[3]
	k0=w[4]
	Y0=w[5]
	complexer[0,midpointt-1] = Rs+1/(1/(Rp+1/Y0/sqrt(k0+ki*freqw))+(Qdl*(ki*freqw)^n))
	complexcat[0,midpointt-1]=real(complexer)
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End

Function setupZunifiedtlineC()
	Make/O/D limits ={{0,0,0,0,0,0,0,0,0,0,0,0,0},{100,1000,1,1e6,.01,1,1e6,.01,1,10000,.01,1,1}}		// make the lower and upper limits
	Make/O/D coefs = {50,500,.9,1e4,.00001,.9,1e4,.00001,100,.9,.00001,.9,.001}			// make the initial coefficients
	make/T/O coeffnames={"Rs","r1","r2","ra","qa","na","rb","qb","nb","rk","qk","nk","L"}	// make a list of coefficient names
	make/O/I/N=(numpnts(coefs)) heldlist=0		// 0 = not held.  1 = held. (during fit)
	string/g currentmodel="ZunifiedtlineC"
	wave/t testsetupvals
	testsetupvals[0]="ZunifiedtlineC"
	make/t/o tempwavenames={"complexer","lam","slam","clam","Za","Zb","ksi"}
	preflight1()
End

Threadsafe Function ZunifiedtlineC(w,complexcat,freqw): fitfunc
	wave complexcat, w, freqw //declare wave names.  Don't edit this
	wave/c complexer, lam, slam, clam, Za, Zb, ksi
	variable/g midpointt
	variable/g/c ki //pull in complex constant
	variable Rs,x1,x2,ra,qa,na,rb,qb,nb,rk,qk,nk,L //declare variable names; you can edit this
	
	Rs=w[0]//	Rs
	x1=w[1]//	r1
	x2=w[2]//	r2
	ra=w[3]//	ra
	qa=w[4]//	qa
	na=w[5]//	na
	rb=w[6]//	rb
	qb=w[7]//	qb
	nb=w[8]//	nb
	rk=w[9]//	rk
	qk=w[10]//	qk
	nk=w[11]//	nk
	L=w[12]//	L

	ksi = Rk/(1 + Rk*Qk*(ki*freqw)^nk);
	lam = Sqrt(ksi/(x1 + x2));
	slam = Sinh(L/lam);
	clam = Cosh(L/lam);
	Za = Ra/(1 + Ra*Qa*(ki*freqw)^na);
	Zb = Rb/(1 + Rb*Qb*(ki*freqw)^nb);
	

	complexer[0,midpointt-1]=Rs+ 1/(x1 + x2)*(lam*(x1 + x2)*slam + (Za + Zb)*clam + 1/(lam*(x1 + x2))*Za*Zb*slam)^-1*(L*lam*x1*x2*(x1 + x2)*slam + x1*(lam*x1*slam + L*x2*clam)*Za + x2*(lam*x2*slam + L*x1*clam)*Zb + 1/(x1 + x2)*(2*x1*x2 + (x1^2 + x2^2)*clam + L/lam*x1*x2*slam)*Za*Zb)
	complexcat[0,midpointt-1]=real(complexer[p])
	complexcat[midpointt, ]=imag(complexer[p-midpointt])
End